@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 27803 2008-09-10 16:07:00Z olivier $
+/*      $Id: compositor.c 29275 2009-01-18 21:45:33Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
         xcompmgr - (c) 2003 Keith Packard
         metacity - (c) 2003, 2004 Red Hat, Inc.
-        xfwm4    - (c) 2005-2008 Olivier Fourdan
+        xfwm4    - (c) 2005-2009 Olivier Fourdan
 
 */
 
@@ -50,19 +50,15 @@
 #include <X11/extensions/Xrender.h>
 
 #ifndef SHADOW_RADIUS
-#define SHADOW_RADIUS   6
+#define SHADOW_RADIUS   12
 #endif /* SHADOW_RADIUS */
 
-#ifndef SHADOW_OPACITY
-#define SHADOW_OPACITY  0.66
-#endif /* SHADOW_OPACITY */
-
 #ifndef SHADOW_OFFSET_X
-#define SHADOW_OFFSET_X (SHADOW_RADIUS * -3 /2)
+#define SHADOW_OFFSET_X (-3 * SHADOW_RADIUS / 2)
 #endif /* SHADOW_OFFSET_X */
 
 #ifndef SHADOW_OFFSET_Y
-#define SHADOW_OFFSET_Y (SHADOW_RADIUS * -5 / 4)
+#define SHADOW_OFFSET_Y (-3 * SHADOW_RADIUS / 2)
 #endif /* SHADOW_OFFSET_Y */
 
 /* Some convenient macros */
@@ -955,9 +951,9 @@ win_extents (CWindow *cw)
         {
             double shadow_opacity;
             shadow_opacity = (double) screen_info->params->frame_opacity
-                                      * SHADOW_OPACITY
-                                      * cw->opacity
-                                      / (NET_WM_OPAQUE * 100.0);
+                           * (screen_info->params->shadow_opacity / 100.0)
+                           * cw->opacity
+                           / (NET_WM_OPAQUE * 100.0);
 
             cw->shadow = shadow_picture (screen_info, shadow_opacity,
                                          cw->attr.width + 2 * cw->attr.border_width,
