@@ -1,4 +1,4 @@
-/*      $Id: placement.c 29029 2008-12-16 22:16:01Z olivier $
+/*      $Id: placement.c 29512 2009-02-18 22:17:27Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -202,7 +202,6 @@ clientConstrainPos (Client * c, gboolean show_full)
     int screen_width, screen_height;
     unsigned int ret;
     GdkRectangle rect;
-    gint monitor_nbr;
     gint min_visible;
 
     g_return_val_if_fail (c != NULL, 0);
@@ -225,8 +224,7 @@ clientConstrainPos (Client * c, gboolean show_full)
     cy = frame_y + (frame_height / 2);
 
     screen_info = c->screen_info;
-    monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
-    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+    myScreenFindMonitorAtPoint (screen_info, cx, cy, &rect);
 
     screen_width = screen_info->width;
     screen_height = screen_info->height;
@@ -650,7 +648,6 @@ clientInitPosition (Client * c)
     Client *c2;
     GdkRectangle rect;
     int full_x, full_y, full_w, full_h, msx, msy;
-    gint monitor_nbr;
     gint n_monitors;
     gboolean place;
     gboolean position;
@@ -664,14 +661,15 @@ clientInitPosition (Client * c)
     position = (c->size->flags & (PPosition | USPosition));
 
     n_monitors = gdk_screen_get_n_monitors (c->screen_info->gscr);
-    monitor_nbr = 0;
     if ((n_monitors > 1) || (screen_info->params->placement_mode == PLACE_MOUSE))
     {
         getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
-        monitor_nbr = find_monitor_at_point (screen_info->gscr, msx, msy);
+        myScreenFindMonitorAtPoint (screen_info, msx, msy, &rect);
+    }
+    else
+    {
+        gdk_screen_get_monitor_geometry (screen_info->gscr, 0, &rect);
     }
-    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
-
     if (position || (c->type & (WINDOW_TYPE_DONT_PLACE | WINDOW_TYPE_DIALOG)) || clientIsTransient (c))
     {
         if (!position && clientIsTransient (c) && (c2 = clientGetTransient (c)))
@@ -684,8 +682,7 @@ clientInitPosition (Client * c)
             {
                 msx = frameX (c) + (frameWidth (c) / 2);
                 msy = frameY (c) + (frameHeight (c) / 2);
-                monitor_nbr = find_monitor_at_point (screen_info->gscr, msx, msy);
-                gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+                myScreenFindMonitorAtPoint (screen_info, msx, msy, &rect);
             }
         }
         if (CONSTRAINED_WINDOW (c))
@@ -761,9 +758,9 @@ clientFill (Client * c, int fill_type)
     unsigned short mask;
     int i, cx, cy, full_x, full_y, full_w, full_h;
     int tmp_x, tmp_y, tmp_w, tmp_h;
-    gint monitor_nbr;
 
     g_return_if_fail (c != NULL);
+    TRACE ("entering clientFill");
 
     if (!CLIENT_CAN_FILL_WINDOW (c))
     {
@@ -797,29 +794,12 @@ clientFill (Client * c, int fill_type)
                 {
                     if ((frameX(c2) + frameWidth(c2)) < frameX(c))
                     {
-                        if (east_neighbour)
-                        {
-                            /* Check if c2 is closer to the client
-                             * then the east neighbour already found
-                             */
-                            if ((frameX(east_neighbour) + frameWidth(east_neighbour)) < (frameX(c2) + frameWidth(c2)))
-                            {
-                                east_neighbour = c2;
-                            }
-                        }
-                        else
-                        {
-                            east_neighbour = c2;
-                        }
-                    }
-                    if ((frameX(c) + frameWidth(c)) < frameX(c2))
-                    {
-                        /* Check if c2 is closer to the client
-                         * then the west neighbour already found
-                         */
                         if (west_neighbour)
                         {
-                            if (frameX(c2) < frameX(west_neighbour))
+                            /* Check if c2 is closer to the client
+                             * then the west neighbour already found
+                             */
+                            if ((frameX(west_neighbour) + frameWidth(west_neighbour)) < (frameX(c2) + frameWidth(c2)))
                             {
                                 west_neighbour = c2;
                             }
@@ -829,6 +809,23 @@ clientFill (Client * c, int fill_type)
                             west_neighbour = c2;
                         }
                     }
+                    if ((frameX(c) + frameWidth(c)) < frameX(c2))
+                    {
+                        /* Check if c2 is closer to the client
+                         * then the west neighbour already found
+                         */
+                        if (east_neighbour)
+                        {
+                            if (frameX(c2) < frameX(east_neighbour))
+                            {
+                                east_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            east_neighbour = c2;
+                        }
+                    }
                 }
             }
 
@@ -888,8 +885,7 @@ clientFill (Client * c, int fill_type)
     cx = tmp_x + (tmp_w / 2);
     cy = tmp_y + (tmp_h / 2);
 
-    monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
-    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+    myScreenFindMonitorAtPoint (screen_info, cx, cy, &rect);
 
     full_x = MAX (screen_info->params->xfwm_margins[STRUTS_LEFT], rect.x);
     full_y = MAX (screen_info->params->xfwm_margins[STRUTS_TOP], rect.y);
@@ -921,44 +917,28 @@ clientFill (Client * c, int fill_type)
      * If not, resize to the largest size available that you just have computed.
      */
 
-    wc.x = frameLeft(c);
-    if (east_neighbour)
-    {
-        wc.x += MAX (frameX(east_neighbour) + frameWidth(east_neighbour), full_x);
-    }
-    else
-    {
-        wc.x += full_x;
-    }
-
-    wc.width = full_x - frameRight(c) - wc.x;
+    wc.x = full_x + frameLeft(c);
     if (west_neighbour)
     {
-        wc.width += MIN (frameX(west_neighbour), full_w);
-    }
-    else
-    {
-        wc.width += full_w;
+        wc.x += MAX (frameX(west_neighbour) + frameWidth(west_neighbour) - full_x, 0);
     }
 
-    wc.y = frameTop(c);
+    wc.width = full_w - frameRight(c) - (wc.x - full_x);
+    if (east_neighbour)
+    {
+        wc.width -= MAX (full_w - (frameX(east_neighbour) - full_x), 0);
+    }
+
+    wc.y = full_y + frameTop(c);
     if (north_neighbour)
     {
-        wc.y += MAX (frameY(north_neighbour) + frameHeight(north_neighbour), full_y);
-    }
-    else
-    {
-        wc.y += full_y;
+        wc.y += MAX (frameY(north_neighbour) + frameHeight(north_neighbour) - full_y, 0);
     }
 
-    wc.height = full_y - frameBottom(c) - wc.y;
+    wc.height = full_h - frameBottom(c) - (wc.y - full_y);
     if (south_neighbour)
     {
-        wc.height += MIN (frameY(south_neighbour), full_h);
-    }
-    else
-    {
-        wc.height += full_h;
+        wc.height -= MAX (full_h - (frameY(south_neighbour) - full_y), 0);
     }
 
     TRACE ("Fill size request: (%d,%d) %dx%d", wc.x, wc.y, wc.width, wc.height);
